// Window1.xaml.cs
#define DEBUG
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Collections.ObjectModel;

namespace PersonBinding {
    public class Person : INotifyPropertyChanged {
        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        protected void FirePropertyChanged(string propertyName) {
            if( this.PropertyChanged != null ) {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        string name;
        public string Name {
            get { return this.name; }
            set {
                this.name = value;
                FirePropertyChanged("Name");
            }
        }

        int age;
        public int Age {
            get { return this.age; }
            set {
                this.age = value;
                FirePropertyChanged("Age");
            }
        }

        public Person() { }
        public Person(string name, int age) {
            this.name = name;
            this.age = age;
        }
    }

    //public class People : List<Person> {
    public class People : ObservableCollection<Person> {
        public People() {
            //Add(new Person("Tom", 9));
            //Add(new Person("John", 10));
            //Add(new Person("Melissa", 36));
        }
    }

    public class RemotePeopleLoader : People {
        //public RemotePeopleLoader() {
        public RemotePeopleLoader(string url1, string url2) {
            //MessageBox.Show(url1, "URL 1:");
            //MessageBox.Show(url2, "URL 2:");
            // Load People from afar
            // NOTE: ain't simulation grand?
            //System.Threading.Thread.Sleep(5000);
            this.Add(new Person("Cathy", 53));
            this.Add(new Person("Sue", 55));
            this.Add(new Person("Nancy", 56));
        }
    }

    public class AgeToForegroundConverter : IValueConverter {
        #region IValueConverter Members
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
            Debug.Assert(targetType == typeof(Brush));

            // DANGER! After 25, it's all down hill...
            int age = int.Parse(value.ToString());
            return (age > 25 ? Brushes.Red : Brushes.Black);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
            // should never be called
            throw new NotImplementedException();
        }
        #endregion
    }

    class PersonSorter : IComparer {
        public int Compare(object x, object y) {
            Person lhs = (Person)x;
            Person rhs = (Person)y;

            // Sort Name ascending and Age descending
            int nameCompare = lhs.Name.CompareTo(rhs.Name);

            if( nameCompare != 0 ) {
                return nameCompare;
            }

            return rhs.Age - lhs.Age;
        }
    }


    public partial class Window1 : Window {
        //Person person = new Person("Tom", 9);

        public Window1() {
            InitializeComponent();

            // Fill initial person fields
            //this.nameTextBox.Text = person.Name;
            //this.ageTextBox.Text = person.Age.ToString();

            // Watch for changes in Tom's properties
            //person.PropertyChanged += person_PropertyChanged;

            // Watch for changes in the controls
            //this.nameTextBox.TextChanged += nameTextBox_TextChanged;
            //this.ageTextBox.TextChanged += ageTextBox_TextChanged;

            // Let the grid know it's data context
            //grid.DataContext = person;

            this.birthdayButton.Click += birthdayButton_Click;
            this.backButton.Click += backButton_Click;
            this.forwardButton.Click += forwardButton_Click;
            this.addButton.Click += addButton_Click;
            this.sortButton.Click += sortButton_Click;
            this.filterButton.Click += filterButton_Click;
        }

        //void person_PropertyChanged(object sender, PropertyChangedEventArgs e) {
        //  switch( e.PropertyName ) {
        //    case "Name":
        //    this.nameTextBox.Text = person.Name;
        //    break;

        //    case "Age":
        //    this.ageTextBox.Text = person.Age.ToString();
        //    break;
        //  }
        //}

        //void nameTextBox_TextChanged(object sender, TextChangedEventArgs e) {
        //  person.Name = nameTextBox.Text;
        //}

        //void ageTextBox_TextChanged(object sender, TextChangedEventArgs e) {
        //  int age = 0;
        //  if( int.TryParse(ageTextBox.Text, out age) ) {
        //    person.Age = age;
        //  }
        //}

        ICollectionView GetFamilyView() {
            //People people = (People)this.FindResource("Family");
            DataSourceProvider ds = (DataSourceProvider)this.FindResource("Family");
            People people = (People)ds.Data;
            return CollectionViewSource.GetDefaultView(people);
        }

        void birthdayButton_Click(object sender, RoutedEventArgs e) {
            ICollectionView view = GetFamilyView();
            Person person = (Person)view.CurrentItem;

            ++person.Age;
            MessageBox.Show(string.Format("Happy Birthday, {0}, age {1}!", person.Name, person.Age), "Birthday");
        }

        void backButton_Click(object sender, RoutedEventArgs e) {
            ICollectionView view = GetFamilyView();
            view.MoveCurrentToPrevious();
            if( view.IsCurrentBeforeFirst ) {
                view.MoveCurrentToFirst();
            }
        }

        void forwardButton_Click(object sender, RoutedEventArgs e) {
            ICollectionView view = GetFamilyView();
            view.MoveCurrentToNext();
            if( view.IsCurrentAfterLast ) {
                view.MoveCurrentToLast();
            }
        }

        void addButton_Click(object sender, RoutedEventArgs e) {
            //People people = (People)this.FindResource("Family");
            DataSourceProvider ds = (DataSourceProvider)this.FindResource("Family");
            People people = (People)ds.Data;
            people.Add(new Person("Chris", 35));
        }

        void sortButton_Click(object sender, RoutedEventArgs e) {
            ListCollectionView view = (ListCollectionView)GetFamilyView();
            //if( view.Sort.Count == 0 ) {
            //  view.Sort.Add(new SortDescription("Name", ListSortDirection.Ascending));
            //  view.Sort.Add(new SortDescription("Age", ListSortDirection.Descending));
            //}
            //else {
            //  view.Sort.Clear();
            //}
            if( view.CustomSort == null ) {
                view.CustomSort = new PersonSorter();
            }
            else {
                view.CustomSort = null;
            }
        }

        void filterButton_Click(object sender, RoutedEventArgs e) {
            ICollectionView view = GetFamilyView();
            if( view.Filter == null ) {
                view.Filter = delegate(object item) { return ((Person)item).Age >= 18; };
            }
            else {
                view.Filter = null;
            }
        }

    }
}

