// Window1.xaml.cs
#define DEBUG
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace PersonBinding {
  public class AgeToForegroundConverter : IValueConverter {
    #region IValueConverter Members
    public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      // DANGER! After 25, it's all down hill...
      int age = int.Parse(value.ToString());
      return (age > 25 ? Brushes.Red : Brushes.Black);
    }

    public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      throw new NotImplementedException();
    }
    #endregion
  }

  class PersonSorter : IComparer {
    public int Compare(object x, object y) {
      XmlElement lhs = (XmlElement)x;
      XmlElement rhs = (XmlElement)y;

      // Sort Name ascending and Age descending
      int nameCompare = lhs.Attributes["Name"].Value.CompareTo(rhs.Attributes["Name"].Value);

      if( nameCompare != 0 ) {
        return nameCompare;
      }

      // TODO: check
      return int.Parse(rhs.Attributes["Age"].Value) - int.Parse(lhs.Attributes["Age"].Value);
    }
  }


  public partial class Window1 : Window {

    public Window1() {
      InitializeComponent();

      this.birthdayButton.Click += birthdayButton_Click;
      this.backButton.Click += backButton_Click;
      this.forwardButton.Click += forwardButton_Click;
      this.addButton.Click += addButton_Click;
      this.sortButton.Click += sortButton_Click;
      this.filterButton.Click += filterButton_Click;
    }

    ICollectionView GetFamilyView() {
      DataSourceProvider ds = (DataSourceProvider)this.FindResource("Family");
      IEnumerable people = (IEnumerable)ds.Data;
      return CollectionViewSource.GetDefaultView(people);
    }

    void birthdayButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();

      XmlElement person = (XmlElement)view.CurrentItem;
      person.SetAttribute("Age", (int.Parse(person.Attributes["Age"].Value) + 1).ToString());
      MessageBox.Show(
        string.Format(
          "Happy Birthday, {0}, age {1}!",
          person.Attributes["Name"].Value,
          person.Attributes["Age"].Value),
        "Birthday");
    }

    void backButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToPrevious();
      if( view.IsCurrentBeforeFirst ) {
        view.MoveCurrentToFirst();
      }
    }

    void forwardButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToNext();
      if( view.IsCurrentAfterLast ) {
        view.MoveCurrentToLast();
      }
    }

    void addButton_Click(object sender, RoutedEventArgs e) {
      XmlDataProvider xds = (XmlDataProvider)this.FindResource("Family");
      XmlElement person = xds.Document.CreateElement("Person");
      person.SetAttribute("Name", "Chris");
      person.SetAttribute("Age", "35");
      xds.Document.ChildNodes[0].AppendChild(person);
    }

    void sortButton_Click(object sender, RoutedEventArgs e) {
      ListCollectionView view = (ListCollectionView)GetFamilyView();
      if( view.CustomSort == null ) {
        view.CustomSort = new PersonSorter();
      }
      else {
        view.CustomSort = null;
      }
    }

    void filterButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      if( view.Filter == null ) {
        view.Filter = delegate(object item) { return int.Parse(((XmlElement)item).Attributes["Age"].Value) >= 18; };
      }
      else {
        view.Filter = null;
      }
    }

  }
}

