// Window1.xaml.cs
#define DEBUG
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Data;
using System.Data.OleDb;

namespace PersonBinding {
  public class AgeToForegroundConverter : IValueConverter {
    #region IValueConverter Members
    public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      // DANGER! After 25, it's all down hill...
      int age = int.Parse(value.ToString());
      return (age > 25 ? Brushes.Red : Brushes.Black);
    }

    public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      throw new NotImplementedException();
    }
    #endregion
  }

  public class AccessFamilyDataSource : IDataSource {
    DataTable table;

    public AccessFamilyDataSource() {
      string mdb = "family.mdb";
      string cs = string.Format("Provider=Microsoft.Jet.OLEDB.4.0; Data Source={0}", mdb);
      OleDbConnection conn = new OleDbConnection(cs);
      OleDbDataAdapter adapter = new OleDbDataAdapter();
      adapter.SelectCommand = new OleDbCommand("select * from people;", conn);
      DataSet dataSet = new DataSet();
      adapter.Fill(dataSet);
      this.table = dataSet.Tables[0];
    }

    #region IDataSource Members
    public object Data { get { return this.table; } }

    public event EventHandler DataChanged;
    public void Refresh() { }
    #endregion
  }

  public partial class Window1 : Window {
    public Window1() {
      InitializeComponent();

      this.birthdayButton.Click += birthdayButton_Click;
      this.backButton.Click += backButton_Click;
      this.forwardButton.Click += forwardButton_Click;
      this.addButton.Click += addButton_Click;
      this.sortButton.Click += sortButton_Click;
      this.filterButton.Click += filterButton_Click;
    }

    ICollectionView GetFamilyView() {
      // DONE: Table is an IListSource, not an IEnumerable
      IDataSource ds = (IDataSource)FindResource("Family");
      ICollectionView view = BindingOperations.GetDefaultView((IListSource)ds.Data);
      return view;
    }

    void birthdayButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      // DONE: Dealing with DataRowViews
      DataRowView person = (DataRowView)view.CurrentItem;
      person["Age"] = (int)person["Age"] + 1;
      MessageBox.Show(
        string.Format(
          "Happy Birthday, {0}, age {1}!",
          person["Name"],
          person["Age"]),
        "Birthday");
    }

    void backButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToPrevious();
      if( view.IsCurrentBeforeFirst ) {
        view.MoveCurrentToFirst();
      }
    }

    void forwardButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToNext();
      if( view.IsCurrentAfterLast ) {
        view.MoveCurrentToLast();
      }
    }

    void addButton_Click(object sender, RoutedEventArgs e) {
      // DONE: Creating a new DataRow (Crashes?!?)
      DataTable table = (DataTable)this.DataContext;
      DataRow person = table.NewRow();
      person["Name"] = "Chris";
      person["Age"] = 35;
      table.Rows.Add(person);
    }

    void sortButton_Click(object sender, RoutedEventArgs e) {
      // Using a BindingListCollectionView
      BindingListCollectionView view = (BindingListCollectionView)GetFamilyView();

      // Sort Name ascending (can only sort on one parameter)
      // (No support for custom sorting)
      if( view.Sort.Count == 0 ) {
        view.Sort.Add(new SortDescription("Name", ListSortDirection.Ascending));
      }
      else {
        view.Sort.Clear();
      }
    }

    void filterButton_Click(object sender, RoutedEventArgs e) {
      // Dealing with DataRowViews
      ICollectionView view = GetFamilyView();
      if( view.Filter == null ) {
        view.Filter = delegate(object item) { return (int)((DataRowView)item)["Age"] >= 18; };
      }
      else {
        view.Filter = null;
      }
    }

  }
}

