// Window1.xaml.cs
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TicTacToe {
  public class PlayerMove {
    string playerName;
    public string PlayerName {
      get { return playerName; }
      set { playerName = value; }
    }

    int moveNumber;
    public int MoveNumber {
      get { return moveNumber; }
      set { moveNumber = value; }
    }

    public PlayerMove(string playerName, int moveNumber) {
      this.playerName = playerName;
      this.moveNumber = moveNumber;
    }
  }

  public partial class Window1 : Window {
    string currentPlayer;
    Button[] cells;
    int moveNumber;

    public Window1() {
      InitializeComponent();

      this.cells = new Button[] { this.cell00, this.cell01, this.cell02, this.cell10, this.cell11, this.cell12, this.cell20, this.cell21, this.cell22 };
      foreach( Button cell in this.cells ) {
        cell.Click += cell_Click;
      }

      NewGame();
    }

    string CurrentPlayer {
      get { return this.currentPlayer; }
      set {
        this.currentPlayer = value;
        this.statusTextBlock.TextContent = "It's your turn, " + this.currentPlayer;
      }
    }

    void NewGame() {
      foreach( Button cell in this.cells ) {
        cell.Content = null;
      }
      CurrentPlayer = "X";
      this.moveNumber = 0;
    }

    void cell_Click(object sender, RoutedEventArgs e) {
      Button button = (Button)sender;

      // Don't let multiple clicks change the player for a cell
      if( button.Content != null ) { return; }

      // Set button content
      //button.Content = CurrentPlayer;
      button.Content = new PlayerMove(this.CurrentPlayer, ++this.moveNumber);

      // Only allow each player to have three moves on at once
      Button[] currentPlayerButtons = Array.FindAll(
            this.cells,
            delegate(Button it) { return it.Content != null && ((PlayerMove)it.Content).PlayerName == this.currentPlayer; });
      if( currentPlayerButtons.Length == 4 ) {
        Button earliestButton = currentPlayerButtons[0];
        Array.ForEach(
          currentPlayerButtons,
          delegate(Button it) {
            if( ((PlayerMove)it.Content).MoveNumber < ((PlayerMove)earliestButton.Content).MoveNumber ) {
              earliestButton = it;
            }
          });
        earliestButton.Content = null;
      }

      // Check for winner
      if( HasWon(CurrentPlayer) ) {
        MessageBox.Show("Winner!", "Game Over");
        NewGame();
        return;
      }
      else if( TieGame() ) {
        MessageBox.Show("No Winner!", "Game Over");
        NewGame();
        return;
      }

      // Switch player
      if( CurrentPlayer == "X" ) {
        //button.Style = (Style)FindResource("XStyle"); 
        CurrentPlayer = "O";
      }
      else {
        //button.Style = (Style)FindResource("OStyle"); 
        CurrentPlayer = "X";
      }

    }

    bool HasWon(string player) {
      Button[,] possibleWins = {
      // row
      { this.cell00, this.cell01, this.cell02, },
      { this.cell10, this.cell11, this.cell12, },
      { this.cell20, this.cell21, this.cell22, },

      // column
      { this.cell00, this.cell10, this.cell20, },
      { this.cell01, this.cell11, this.cell21, },
      { this.cell02, this.cell12, this.cell22, },

      // diagonal
      { this.cell00, this.cell11, this.cell22, },
      { this.cell02, this.cell11, this.cell20, },
    };

      for( int i = 0; i != possibleWins.GetLength(0); ++i ) {
        bool hasWon = true;
        for( int j = 0; j != possibleWins.GetLength(1); ++j ) {
          if( possibleWins[i, j].Content == null || ((PlayerMove)possibleWins[i, j].Content).PlayerName != player ) {
            hasWon = false;
            break;
          }
        }
        if( hasWon ) { return true; }
      }

      return false;
    }

    bool TieGame() {
      foreach( Button cell in this.cells ) {
        if( cell.Content == null ) {
          return false;
        }
      }
      return true;
    }
  }
}
